"use strict";
/*
 * Copyright (C) 2020-2022 UBports Foundation <info@ubports.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAndroidToolBaseDir = getAndroidToolBaseDir;
exports.getAndroidToolPath = getAndroidToolPath;

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _dirname2 = _interopRequireDefault(require("./dirname.cjs"));

// HACK: Currently required for CommonJS compatibility.

/**
 * Get base directory path for debugging
 *
 * @returns {String} base directory path
 */
function getAndroidToolBaseDir() {
  return _path["default"].join(_dirname2["default"], "..", "dist", process.platform, normalizedArch(process.arch));
}
/**
 * Group architectures together
 *
 * @param {String} arch - architecture returned by process.arch
 * @returns {String} compatible packaged architecture
 */


function normalizedArch(arch) {
  switch (arch) {
    case "ia32":
    case "x32":
    case "x64":
      return "x86";

    case "arm":
    case "arm64":
      return "arm";

    default:
      return arch;
  }
}
/**
 * Returns path to an android tool. If the env var USE_SYSTEM_<tool> where <tool> is the capitalized name of the tool is set, the function will return the native tool for that tool accordingly. If the env var USE_NATIVE_SYSTEM_TOOLS is said, all tools will return the native tools.
 *
 * @param {String} tool - tool to return
 * @returns {String} path to tool or native command
 */


function getAndroidToolPath(tool) {
  var optimistic = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

  var _native = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  if (!tool || typeof tool !== "string") {
    throw new Error("Invalid argument: ".concat(tool));
  }

  try {
    if (_native.all || process.env.USE_SYSTEM_TOOLS || _native[tool] || process.env["USE_SYSTEM_".concat(tool === null || tool === void 0 ? void 0 : tool.toUpperCase())]) {
      return tool;
    }

    var assumedPath = _path["default"].join(getAndroidToolBaseDir(), process.platform === "win32" ? "".concat(tool, ".exe") : tool);

    if (_fs["default"].existsSync(assumedPath)) {
      return assumedPath;
    } else if (optimistic) {
      return tool;
    } else {
      throw new Error("No binary of ".concat(tool, " for ").concat(process.arch, " ").concat(process.platform));
    }
  } catch (error) {
    if (optimistic) return tool;else throw new Error("Failed to get tool: ".concat(error));
  }
}